@@ -17,6 +17,8 @@
 
 #include "base/ZoomConstraint.h"
 
+namespace sv {
+
 class RelativelyFineZoomConstraint : virtual public ZoomConstraint
 {
 public:
@@ -25,5 +27,7 @@ public:
 	const override;
 };
 
+} // end namespace sv
+
 #endif
 
