@@ -20,6 +20,8 @@
 
 #include <stdlib.h>
 
+namespace sv {
+
 class WaveFileModel : public RangeSummarisableTimeValueModel
 {
     Q_OBJECT
@@ -27,22 +29,28 @@ class WaveFileModel : public RangeSummarisableTimeValueModel
 public:
     virtual ~WaveFileModel();
 
+    QString getValueUnit() const override {
+        return "V";
+    }
+    
     virtual sv_frame_t getFrameCount() const = 0;
-    virtual int getChannelCount() const = 0;
-    virtual sv_samplerate_t getSampleRate() const = 0;
-    virtual sv_samplerate_t getNativeRate() const = 0;
-
-    virtual QString getTitle() const = 0;
-    virtual QString getMaker() const = 0;
-    virtual QString getLocation() const = 0;
-
-    virtual sv_frame_t getStartFrame() const = 0;
-    virtual sv_frame_t getEndFrame() const = 0;
-
     virtual void setStartFrame(sv_frame_t startFrame) = 0;
+    
+    int getChannelCount() const override = 0;
+    sv_samplerate_t getSampleRate() const override = 0;
+    sv_samplerate_t getNativeRate() const override = 0;
+
+    QString getTitle() const override = 0;
+    QString getMaker() const override = 0;
+    QString getLocation() const override = 0;
+
+    sv_frame_t getStartFrame() const override = 0;
+    sv_frame_t getTrueEndFrame() const override = 0;
 
 protected:
     WaveFileModel() { } // only accessible from subclasses
 };    
 
+} // end namespace sv
+
 #endif
