@@ -20,6 +20,8 @@
 #include <cmath>
 
 #include "base/Pitch.h"
+#include "base/Debug.h"
+#include "base/UnitDatabase.h"
 
 #include "LayerGeometryProvider.h"
 #include "HorizontalScaleProvider.h"
@@ -27,12 +29,13 @@
 #include <iostream>
 using namespace std;
 
+namespace sv {
+
 void
 PianoScale::paintPianoVertical(LayerGeometryProvider *v,
                                QPainter &paint,
                                QRect r,
-                               double minf,
-                               double maxf)
+                               const CoordinateScale &scale)
 {
     int x0 = r.x(), y0 = r.y(), x1 = r.x() + r.width(), y1 = r.y() + r.height();
 
@@ -41,10 +44,14 @@ PianoScale::paintPianoVertical(LayerGeometryProvider *v,
     int py = y1, ppy = y1;
     paint.setBrush(paint.pen().color());
 
+    if (UnitDatabase::asCommonUnit(scale.getUnit()) != "Hz") {
+        SVDEBUG << "WARNING: PianoScale::paintPianoVertical: CoordinateScale unit is not Hz (it is: \"" << scale.getUnit() << "\")" << endl;
+    }
+    
     for (int i = 0; i < 128; ++i) {
 
         double f = Pitch::getFrequencyForPitch(i);
-        int y = int(lrint(v->getYForFrequency(f, minf, maxf, true)));
+        int y = scale.getCoordForValueRounded(v, f);
 
         if (y < y0 - 2) break;
         if (y > y1 + 2) {
@@ -58,7 +65,7 @@ PianoScale::paintPianoVertical(LayerGeometryProvider *v,
             QColor col = Qt::gray;
             if (i == 61) { // filling middle C
                 col = Qt::blue;
-                col = col.light(150);
+                col = col.lighter(150);
             }
             if (ppy - y > 2) {
                 paint.fillRect(x0 + 1,
@@ -129,7 +136,7 @@ PianoScale::paintPianoHorizontal(LayerGeometryProvider *v,
             QColor col = Qt::gray;
             if (i == 61) { // filling middle C
                 col = Qt::blue;
-                col = col.light(150);
+                col = col.lighter(150);
             }
             if (x - ppx > 2) {
                 paint.fillRect((px + ppx) / 2 + 1,
@@ -159,3 +166,5 @@ PianoScale::paintPianoHorizontal(LayerGeometryProvider *v,
 }
 
 
+} // end namespace sv
+
