@@ -20,6 +20,7 @@
 #include "base/Profiler.h"
 #include "base/RangeMapper.h"
 #include "base/Strings.h"
+#include "base/ScaleTickIntervals.h"
 
 #include "ColourDatabase.h"
 #include "PaintAssistant.h"
@@ -27,6 +28,7 @@
 #include "data/model/WaveformOversampler.h"
 
 #include <QPainter>
+#include <QPainterPath>
 #include <QPixmap>
 #include <QTextStream>
 
@@ -36,17 +38,22 @@
 //#define DEBUG_WAVEFORM_PAINT 1
 //#define DEBUG_WAVEFORM_PAINT_BY_PIXEL 1
 
+namespace sv {
+
 using std::vector;
 
+double
+WaveformLayer::m_dBMin = -80.0;
 
 WaveformLayer::WaveformLayer() :
     SingleColourLayer(),
-    m_model(nullptr),
     m_gain(1.0f),
     m_autoNormalize(false),
+    m_oversampling(true),
     m_showMeans(true),
     m_channelMode(SeparateChannels),
     m_channel(-1),
+    m_channelCount(0),
     m_scale(LinearScale),
     m_middleLineHeight(0.5),
     m_aggressive(false),
@@ -60,29 +67,50 @@ WaveformLayer::~WaveformLayer()
     delete m_cache;
 }
 
-void
-WaveformLayer::setModel(const RangeSummarisableTimeValueModel *model)
+const ZoomConstraint *
+WaveformLayer::getZoomConstraint() const
 {
+    auto model = ModelById::get(m_model);
+    if (model) return model->getZoomConstraint();
+    else return nullptr;
+}
+
+void
+WaveformLayer::setModel(ModelId modelId)
+{
+    auto oldModel = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    auto newModel = ModelById::getAs<RangeSummarisableTimeValueModel>(modelId);
+
+    if (!modelId.isNone() && !newModel) {
+        throw std::logic_error("Not a RangeSummarisableTimeValueModel");
+    }
+
+    if (m_model == modelId) return;
+    m_model = modelId;
+
+    // NB newModel may legitimately be null
+    
+    m_cacheValid = false;
+    
     bool channelsChanged = false;
     if (m_channel == -1) {
-        if (!m_model) {
-            if (model) {
+        if (!oldModel) {
+            if (newModel) {
                 channelsChanged = true;
             }
         } else {
-            if (model &&
-                m_model->getChannelCount() != model->getChannelCount()) {
+            if (newModel &&
+                oldModel->getChannelCount() != newModel->getChannelCount()) {
                 channelsChanged = true;
             }
         }
     }
 
-    m_model = model;
-    m_cacheValid = false;
-    if (!m_model || !m_model->isOK()) return;
-
-    connectSignals(m_model);
-
+    if (newModel) {
+        m_channelCount = newModel->getChannelCount();
+        connectSignals(m_model);
+    }
+        
     emit modelReplaced();
 
     if (channelsChanged) emit layerParametersChanged();
@@ -95,8 +123,8 @@ WaveformLayer::getProperties() const
     list.push_back("Scale");
     list.push_back("Gain");
     list.push_back("Normalize Visible Area");
-
-    if (m_model && m_model->getChannelCount() > 1 && m_channel == -1) {
+    list.push_back("Show Oversampling");
+    if (m_channelCount > 1 && m_channel == -1) {
         list.push_back("Channels");
     }
 
@@ -109,6 +137,7 @@ WaveformLayer::getPropertyLabel(const PropertyName &name) const
     if (name == "Scale") return tr("Scale");
     if (name == "Gain") return tr("Gain");
     if (name == "Normalize Visible Area") return tr("Normalize Visible Area");
+    if (name == "Show Oversampling") return tr("Interpolate");
     if (name == "Channels") return tr("Channels");
     return SingleColourLayer::getPropertyLabel(name);
 }
@@ -117,6 +146,7 @@ QString
 WaveformLayer::getPropertyIconName(const PropertyName &name) const
 {
     if (name == "Normalize Visible Area") return "normalise";
+    if (name == "Show Oversampling") return "interpolate";
     return "";
 }
 
@@ -125,6 +155,7 @@ WaveformLayer::getPropertyType(const PropertyName &name) const
 {
     if (name == "Gain") return RangeProperty;
     if (name == "Normalize Visible Area") return ToggleProperty;
+    if (name == "Show Oversampling") return ToggleProperty;
     if (name == "Channels") return ValueProperty;
     if (name == "Scale") return ValueProperty;
     return SingleColourLayer::getPropertyType(name);
@@ -135,6 +166,7 @@ WaveformLayer::getPropertyGroupName(const PropertyName &name) const
 {
     if (name == "Gain" ||
         name == "Normalize Visible Area" ||
+        name == "Show Oversampling" ||
         name == "Scale") return tr("Scale");
     return QString();
 }
@@ -156,7 +188,7 @@ WaveformLayer::getPropertyRangeAndValue(const PropertyName &name,
         *max = 50;
         *deflt = 0;
 
-        val = int(lrint(log10(m_gain) * 20.0));
+        val = int(round(AudioLevel::voltage_to_dB(m_gain)));
         if (val < *min) val = *min;
         if (val > *max) val = *max;
 
@@ -165,6 +197,11 @@ WaveformLayer::getPropertyRangeAndValue(const PropertyName &name,
         val = (m_autoNormalize ? 1 : 0);
         *deflt = 0;
 
+    } else if (name == "Show Oversampling") {
+
+        val = (m_oversampling ? 1 : 0);
+        *deflt = 1;
+
     } else if (name == "Channels") {
 
         *min = 0;
@@ -225,9 +262,11 @@ void
 WaveformLayer::setProperty(const PropertyName &name, int value)
 {
     if (name == "Gain") {
-        setGain(float(pow(10, float(value)/20.0)));
+        setGain(AudioLevel::dB_to_voltage(value));
     } else if (name == "Normalize Visible Area") {
         setAutoNormalize(value ? true : false);
+    } else if (name == "Show Oversampling") {
+        setShowOversampledLines(value ? true : false);
     } else if (name == "Channels") {
         if (value == 1) setChannelMode(MixChannels);
         else if (value == 2) setChannelMode(MergeChannels);
@@ -263,6 +302,15 @@ WaveformLayer::setAutoNormalize(bool autoNormalize)
     emit layerParametersChanged();
 }
 
+void
+WaveformLayer::setShowOversampledLines(bool oversample)
+{
+    if (m_oversampling == oversample) return;
+    m_oversampling = oversample;
+    m_cacheValid = false;
+    emit layerParametersChanged();
+}
+
 void
 WaveformLayer::setShowMeans(bool showMeans)
 {
@@ -323,37 +371,98 @@ int
 WaveformLayer::getCompletion(LayerGeometryProvider *) const
 {
     int completion = 100;
-    if (!m_model || !m_model->isOK()) return completion;
-    if (m_model->isReady(&completion)) return 100;
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model || !model->isOK()) return completion;
+    if (model->isReady(&completion)) return 100;
     return completion;
 }
 
-bool
-WaveformLayer::getValueExtents(double &min, double &max,
-                               bool &, QString &unit) const
+Layer::ScaleExtents
+WaveformLayer::getVerticalExtents() const
 {
-    if (m_scale == LinearScale) {
-        min = 0.0;
-        max = 1.0;
-        unit = "V";
-    } else if (m_scale == MeterScale) {
-        return false; //!!!
-    } else {
-        min = AudioLevel::multiplier_to_dB(0.0);
-        max = AudioLevel::multiplier_to_dB(1.0);
-        unit = "dB";
+    CoordinateScale scale(CoordinateScale::Direction::Vertical,
+                          "V", m_scale == dBScale, -1.0, 1.0);
+
+    bool alignable = true;
+    
+    if (m_scale == MeterScale) {
+        alignable = false;
     }
-    return true;
+    
+    if (m_channelCount > 1) {
+        if (m_channelMode == SeparateChannels) {
+            alignable = false;
+        }
+        if (m_channelMode == MergeChannels && m_scale != LinearScale) {
+            alignable = false;
+        }
+    }
+
+    if (alignable && m_scale == dBScale) {
+        scale = scale.withDisplayExtents
+            (AudioLevel::dB_to_voltage(m_dBMin), 1.0);
+    }
+
+    //!!! display extents not set based on gain (but then
+    //!!! getDisplayExtents doesn't handle it either?)
+
+    return {
+        alignable ? ScaleApplication::Normal : ScaleApplication::Personal,
+        scale
+    };
+}
+
+bool
+WaveformLayer::getDisplayExtents(double &min, double &max) const
+{
+    // If we have a single channel visible and either linear or log
+    // (dB) scale, then we have a continuous scale that runs from -1
+    // to 1 or -dBMin to 0 and we can offer it as an alignment target
+    // for other layers with the same unit. We can also do this in
+    // butterfly mode, but only with linear scale. Otherwise no.
+
+    if (m_scale == MeterScale) {
+        return false;
+    }
+    
+    if (m_channelCount > 1) {
+        if (m_channelMode == SeparateChannels) {
+            return false;
+        }
+        if (m_channelMode == MergeChannels && m_scale != LinearScale) {
+            return false;
+        }
+    }
+
+    if (m_scale == LinearScale) {
+        max = 1.0;
+        min = -1.0;
+        return true;
+    }
+
+    if (m_scale == dBScale) {
+        max = 1.0;
+        min = AudioLevel::dB_to_voltage(m_dBMin);
+        return true;
+    }
+
+    return false;
 }
 
 double
 WaveformLayer::dBscale(double sample, int m) const
 {
     if (sample < 0.0) return dBscale(-sample, m);
-    double dB = AudioLevel::multiplier_to_dB(sample);
-    if (dB < -50.0) return 0;
+    double dB = AudioLevel::voltage_to_dB(sample);
+    if (dB < m_dBMin) return 0;
     if (dB > 0.0) return m;
-    return ((dB + 50.0) * m) / 50.0;
+    return ((dB - m_dBMin) * m) / (-m_dBMin);
+}
+
+double
+WaveformLayer::dBscaleMeter(double sample, int m) const
+{
+    return AudioLevel::voltage_to_fader(sample, m, AudioLevel::Scale::Preview);
 }
 
 int
@@ -361,9 +470,7 @@ WaveformLayer::getChannelArrangement(int &min, int &max,
                                      bool &merging, bool &mixing)
     const
 {
-    if (!m_model || !m_model->isOK()) return 0;
-
-    int channels = m_model->getChannelCount();
+    int channels = m_channelCount;
     if (channels == 0) return 0;
 
     int rawChannels = channels;
@@ -384,7 +491,17 @@ WaveformLayer::getChannelArrangement(int &min, int &max,
         channels = 1;
     }
 
-    merging = (m_channelMode == MergeChannels && rawChannels > 1);
+    // "Merging" -> "butterfly mode" - use +ve side of "waveform" for
+    // channel 0 and -ve side for channel 1. If we only have one
+    // channel, we still do this but just duplicate channel 0 onto
+    // channel 1 - this is the only way to get a classic-looking
+    // waveform with meter or db scale from a single-channel file,
+    // although it isn't currently exposed in the SV UI
+    merging = (m_channelMode == MergeChannels);
+
+    // "Mixing" -> produce a single waveform from the mean of the
+    // channels. Unlike merging, this really does only make sense if
+    // we have >1 channel.
     mixing = (m_channelMode == MixChannels && rawChannels > 1);
 
 //    SVDEBUG << "WaveformLayer::getChannelArrangement: min " << min << ", max " << max << ", merging " << merging << ", channels " << channels << endl;
@@ -406,6 +523,9 @@ WaveformLayer::getSourceFramesForX(LayerGeometryProvider *v,
                                    int x, int modelZoomLevel,
                                    sv_frame_t &f0, sv_frame_t &f1) const
 {
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model) return false;
+    
     sv_frame_t viewFrame = v->getFrameForX(x);
     if (viewFrame < 0) {
         f0 = 0;
@@ -417,7 +537,7 @@ WaveformLayer::getSourceFramesForX(LayerGeometryProvider *v,
     f0 = f0 / modelZoomLevel;
     f0 = f0 * modelZoomLevel;
 
-    if (v->getZoomLevel().zone == ZoomLevel::PixelsPerFrame) {
+    if (v->getRoundedZoomLevel().zone == ZoomLevel::PixelsPerFrame) {
         f1 = f0 + 1;
     } else {
         viewFrame = v->getFrameForX(x + 1);
@@ -426,17 +546,20 @@ WaveformLayer::getSourceFramesForX(LayerGeometryProvider *v,
         f1 = f1 * modelZoomLevel;
     }
     
-    return (f0 < m_model->getEndFrame());
+    return (f0 < model->getEndFrame());
 }
 
 float
 WaveformLayer::getNormalizeGain(LayerGeometryProvider *v, int channel) const
 {
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model) return 0.f;
+    
     sv_frame_t startFrame = v->getStartFrame();
     sv_frame_t endFrame = v->getEndFrame();
 
-    sv_frame_t modelStart = m_model->getStartFrame();
-    sv_frame_t modelEnd = m_model->getEndFrame();
+    sv_frame_t modelStart = model->getStartFrame();
+    sv_frame_t modelEnd = model->getEndFrame();
     
     sv_frame_t rangeStart, rangeEnd;
             
@@ -450,7 +573,7 @@ WaveformLayer::getNormalizeGain(LayerGeometryProvider *v, int channel) const
     if (rangeEnd < rangeStart) rangeEnd = rangeStart;
 
     RangeSummarisableTimeValueModel::Range range =
-        m_model->getSummary(channel, rangeStart, rangeEnd - rangeStart);
+        model->getSummary(channel, rangeStart, rangeEnd - rangeStart);
 
     int minChannel = 0, maxChannel = 0;
     bool mergingChannels = false, mixingChannels = false;
@@ -459,11 +582,13 @@ WaveformLayer::getNormalizeGain(LayerGeometryProvider *v, int channel) const
                                 mergingChannels, mixingChannels);
 
     if (mergingChannels || mixingChannels) {
-        RangeSummarisableTimeValueModel::Range otherRange =
-            m_model->getSummary(1, rangeStart, rangeEnd - rangeStart);
-        range.setMax(std::max(range.max(), otherRange.max()));
-        range.setMin(std::min(range.min(), otherRange.min()));
-        range.setAbsmean(std::min(range.absmean(), otherRange.absmean()));
+        if (m_channelCount > 1) {
+            RangeSummarisableTimeValueModel::Range otherRange =
+                model->getSummary(1, rangeStart, rangeEnd - rangeStart);
+            range.setMax(std::max(range.max(), otherRange.max()));
+            range.setMin(std::min(range.min(), otherRange.min()));
+            range.setAbsmean(std::min(range.absmean(), otherRange.absmean()));
+        }
     }
 
     return float(1.0 / std::max(fabs(range.max()), fabs(range.min())));
@@ -472,14 +597,15 @@ WaveformLayer::getNormalizeGain(LayerGeometryProvider *v, int channel) const
 void
 WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect) const
 {
-    if (!m_model || !m_model->isOK()) {
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model || !model->isOK()) {
         return;
     }
   
-    ZoomLevel zoomLevel = v->getZoomLevel();
+    ZoomLevel zoomLevel = v->getRoundedZoomLevel();
+    Profiler profiler("WaveformLayer::paint", true);
 
 #ifdef DEBUG_WAVEFORM_PAINT
-    Profiler profiler("WaveformLayer::paint", true);
     SVCERR << "WaveformLayer::paint (" << rect.x() << "," << rect.y()
               << ") [" << rect.width() << "x" << rect.height() << "]: zoom " << zoomLevel << endl;
 #endif
@@ -537,7 +663,8 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
         paint = &viewPainter;
     }
 
-    paint->setRenderHint(QPainter::Antialiasing, true);
+    paint->setRenderHint(QPainter::Antialiasing,
+                         zoomLevel.zone == ZoomLevel::PixelsPerFrame);
 
     if (m_middleLineHeight != 0.5) {
         paint->save();
@@ -576,7 +703,7 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
     if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
         desiredBlockSize = zoomLevel.level;
     }
-    int blockSize = m_model->getSummaryBlockSize(desiredBlockSize);
+    int blockSize = model->getSummaryBlockSize(desiredBlockSize);
 
     sv_frame_t frame0;
     sv_frame_t frame1;
@@ -601,7 +728,7 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
 
     RangeVec ranges;
 
-    if (v->getZoomLevel().zone == ZoomLevel::FramesPerPixel) {
+    if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
         getSummaryRanges(minChannel, maxChannel,
                          mixingChannels || mergingChannels,
                          frame0, frame1,
@@ -610,7 +737,7 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
         getOversampledRanges(minChannel, maxChannel,
                              mixingChannels || mergingChannels,
                              frame0, frame1,
-                             v->getZoomLevel().level, ranges);
+                             zoomLevel.level, ranges);
     }
 
     if (!ranges.empty()) {
@@ -625,7 +752,7 @@ WaveformLayer::paint(LayerGeometryProvider *v, QPainter &viewPainter, QRect rect
     }
 
     if (m_aggressive) {
-        if (m_model->isReady() && rect == v->getPaintRect()) {
+        if (model->isReady() && rect == v->getPaintRect()) {
             m_cacheValid = true;
             m_cacheZoomLevel = zoomLevel;
         }
@@ -642,9 +769,12 @@ WaveformLayer::getSummaryRanges(int minChannel, int maxChannel,
                                 int blockSize, RangeVec &ranges)
     const
 {
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model) return;
+    
     for (int ch = minChannel; ch <= maxChannel; ++ch) {
         ranges.push_back({});
-        m_model->getSummaries(ch, frame0, frame1 - frame0,
+        model->getSummaries(ch, frame0, frame1 - frame0,
                               ranges[ch - minChannel], blockSize);
 #ifdef DEBUG_WAVEFORM_PAINT
             SVCERR << "channel " << ch << ": " << ranges[ch - minChannel].size() << " ranges from " << frame0 << " to " << frame1 << " at zoom level " << blockSize << endl;
@@ -654,10 +784,12 @@ WaveformLayer::getSummaryRanges(int minChannel, int maxChannel,
     if (mixingOrMerging) {
         if (minChannel != 0 || maxChannel != 0) {
             throw std::logic_error("Internal error: min & max channels should be 0 when merging or mixing all channels");
-        } else if (m_model->getChannelCount() > 1) {
+        } else if (m_channelCount > 1) {
             ranges.push_back({});
-            m_model->getSummaries
+            model->getSummaries
                 (1, frame0, frame1 - frame0, ranges[1], blockSize);
+        } else {
+            ranges.push_back(ranges[0]);
         }
     }
 }
@@ -669,16 +801,25 @@ WaveformLayer::getOversampledRanges(int minChannel, int maxChannel,
                                     int oversampleBy, RangeVec &ranges)
     const
 {
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model) return;
+    
     if (mixingOrMerging) {
         if (minChannel != 0 || maxChannel != 0) {
             throw std::logic_error("Internal error: min & max channels should be 0 when merging or mixing all channels");
         }
-        if (m_model->getChannelCount() > 1) {
+        if (m_channelCount > 1) {
             // call back on self for the individual channels with
             // mixingOrMerging false
             getOversampledRanges
                 (0, 1, false, frame0, frame1, oversampleBy, ranges);
             return;
+        } else {
+            // call back on self for a single channel, then duplicate
+            getOversampledRanges
+                (0, 0, false, frame0, frame1, oversampleBy, ranges);
+            ranges.push_back(ranges[0]);
+            return;
         }
     }
     
@@ -686,8 +827,8 @@ WaveformLayer::getOversampledRanges(int minChannel, int maxChannel,
     // sample rate, not the oversampled rate
 
     sv_frame_t tail = 16;
-    sv_frame_t startFrame = m_model->getStartFrame();
-    sv_frame_t endFrame = m_model->getEndFrame();
+    sv_frame_t startFrame = model->getStartFrame();
+    sv_frame_t endFrame = model->getEndFrame();
 
     sv_frame_t rf0 = frame0 - tail;
     if (rf0 < startFrame) {
@@ -706,11 +847,12 @@ WaveformLayer::getOversampledRanges(int minChannel, int maxChannel,
     
     for (int ch = minChannel; ch <= maxChannel; ++ch) {
         floatvec_t oversampled = WaveformOversampler::getOversampledData
-            (m_model, ch, frame0, frame1 - frame0, oversampleBy);
+            (*model, ch, frame0, frame1 - frame0, oversampleBy);
         RangeSummarisableTimeValueModel::RangeBlock rr;
         for (float v: oversampled) {
             RangeSummarisableTimeValueModel::Range r;
             r.sample(v);
+            r.setAbsmean(fabsf(v));
             rr.push_back(r);
         }
         ranges.push_back(rr);
@@ -738,6 +880,9 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
                             sv_frame_t frame1)
     const
 {
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model) return;
+    
     int x0 = rect.left();
     int y0 = rect.top();
 
@@ -759,9 +904,9 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
     if (midColour == Qt::black) {
         midColour = Qt::gray;
     } else if (v->hasLightBackground()) {
-        midColour = midColour.light(150);
+        midColour = midColour.lighter(150);
     } else {
-        midColour = midColour.light(50);
+        midColour = midColour.lighter(50);
     }
 
     double gain = m_effectiveGains[ch];
@@ -802,6 +947,8 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
 
     bool firstPoint = true;
     double prevRangeBottom = 0, prevRangeTop = 0;
+
+    auto zoomLevel = v->getRoundedZoomLevel();
     
     for (int x = x0; x <= x1; ++x) {
 
@@ -810,7 +957,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
 
         bool showIndividualSample = false;
         
-        if (v->getZoomLevel().zone == ZoomLevel::FramesPerPixel) {
+        if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
             if (!getSourceFramesForX(v, x, blockSize, f0, f1)) {
                 continue;
             }
@@ -818,7 +965,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
             i0 = (f0 - frame0) / blockSize;
             i1 = (f1 - frame0) / blockSize;
         } else {
-            int oversampleBy = v->getZoomLevel().level;
+            int oversampleBy = zoomLevel.level;
             f0 = f1 = v->getFrameForX(x);
             int xf0 = v->getXForFrame(f0);
             showIndividualSample = (x == xf0);
@@ -836,7 +983,7 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
 #endif
 
         if (i1 > i0 + 1) {
-            SVCERR << "WaveformLayer::paint: ERROR: i1 " << i1 << " > i0 " << i0 << " plus one (zoom = " << v->getZoomLevel() << ", model zoom = " << blockSize << ")" << endl;
+            SVCERR << "WaveformLayer::paint: ERROR: i1 " << i1 << " > i0 " << i0 << " plus one (zoom = " << zoomLevel << ", model zoom = " << blockSize << ")" << endl;
         }
 
         const auto &r = ranges[rangeix];
@@ -919,25 +1066,18 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
 
         case MeterScale:
             if (!mergingChannels) {
-                double r0 = std::abs(AudioLevel::multiplier_to_preview
-                                 (range.min() * gain, m));
-                double r1 = std::abs(AudioLevel::multiplier_to_preview
-                                 (range.max() * gain, m));
+                double r0 = std::abs(dBscaleMeter(range.min() * gain, m));
+                double r1 = std::abs(dBscaleMeter(range.max() * gain, m));
                 rangeTop = std::max(r0, r1);
                 meanTop = std::min(r0, r1);
                 if (mixingChannels) rangeBottom = meanTop;
-                else rangeBottom = AudioLevel::multiplier_to_preview
-                         (range.absmean() * gain, m);
+                else rangeBottom = dBscaleMeter(range.absmean() * gain, m);
                 meanBottom  = rangeBottom;
             } else {
-                rangeBottom = -AudioLevel::multiplier_to_preview
-                    (range.min() * gain, m);
-                rangeTop = AudioLevel::multiplier_to_preview
-                    (range.max() * gain, m);
-                meanBottom = -AudioLevel::multiplier_to_preview
-                    (range.absmean() * gain, m);
-                meanTop = AudioLevel::multiplier_to_preview
-                    (range.absmean() * gain, m);
+                rangeBottom = -dBscaleMeter(range.min() * gain, m);
+                rangeTop = dBscaleMeter(range.max() * gain, m);
+                meanBottom = -dBscaleMeter(range.absmean() * gain, m);
+                meanTop = dBscaleMeter(range.absmean() * gain, m);
             }
             break;
         }
@@ -1029,36 +1169,41 @@ WaveformLayer::paintChannel(LayerGeometryProvider *v,
     }
 
     double penWidth = 1.0;
-    if (v->getZoomLevel().zone == ZoomLevel::FramesPerPixel) {
+    if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
         penWidth = 0.0;
     }
     
-    if (m_model->isReady()) {
+    if (model->isReady()) {
         paint->setPen(QPen(baseColour, penWidth));
     } else {
         paint->setPen(QPen(midColour, penWidth));
     }
-    paint->drawPath(waveformPath);
 
-    if (!clipPath.isEmpty()) {
-        paint->save();
-        paint->setPen(QPen(ColourDatabase::getInstance()->
-                           getContrastingColour(m_colour), penWidth));
-        paint->drawPath(clipPath);
-        paint->restore();
-    }
+    if (zoomLevel.zone == ZoomLevel::FramesPerPixel ||
+        m_oversampling) {
+        
+        paint->drawPath(waveformPath);
 
-    if (!meanPath.isEmpty()) {
-        paint->save();
-        paint->setPen(QPen(midColour, penWidth));
-        paint->drawPath(meanPath);
-        paint->restore();
+        if (!clipPath.isEmpty()) {
+            paint->save();
+            paint->setPen(QPen(ColourDatabase::getInstance()->
+                               getContrastingColour(m_colour), penWidth));
+            paint->drawPath(clipPath);
+            paint->restore();
+        }
+
+        if (!meanPath.isEmpty()) {
+            paint->save();
+            paint->setPen(QPen(midColour, penWidth));
+            paint->drawPath(meanPath);
+            paint->restore();
+        }
     }
     
     if (!individualSamplePoints.empty()) {
         double sz = v->scaleSize(2.0);
-        if (v->getZoomLevel().zone == ZoomLevel::PixelsPerFrame) {
-            if (v->getZoomLevel().level < 10) {
+        if (zoomLevel.zone == ZoomLevel::PixelsPerFrame) {
+            if (zoomLevel.level < 10) {
                 sz = v->scaleSize(1.2);
             }
         }
@@ -1103,11 +1248,11 @@ WaveformLayer::paintChannelScaleGuides(LayerGeometryProvider *v,
                 break;
 
             case MeterScale:
-                val = AudioLevel::dB_to_multiplier(meterdbs[i]) * gain;
+                val = AudioLevel::dB_to_voltage(meterdbs[i]) * gain;
                 break;
 
             case dBScale:
-                val = AudioLevel::dB_to_multiplier(-(10*n) + i * 10) * gain;
+                val = AudioLevel::dB_to_voltage(-(10*n) + i * 10) * gain;
                 break;
             }
 
@@ -1136,24 +1281,25 @@ WaveformLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) cons
 {
     int x = pos.x();
 
-    if (!m_model || !m_model->isOK()) return "";
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model || !model->isOK()) return "";
 
-    ZoomLevel zoomLevel = v->getZoomLevel();
+    ZoomLevel zoomLevel = v->getRoundedZoomLevel();
 
     int desiredBlockSize = 1;
     if (zoomLevel.zone == ZoomLevel::FramesPerPixel) {
         desiredBlockSize = zoomLevel.level;
     }
 
-    int blockSize = m_model->getSummaryBlockSize(desiredBlockSize);
+    int blockSize = model->getSummaryBlockSize(desiredBlockSize);
 
     sv_frame_t f0, f1;
     if (!getSourceFramesForX(v, x, blockSize, f0, f1)) return "";
     
     QString text;
 
-    RealTime rt0 = RealTime::frame2RealTime(f0, m_model->getSampleRate());
-    RealTime rt1 = RealTime::frame2RealTime(f1, m_model->getSampleRate());
+    RealTime rt0 = RealTime::frame2RealTime(f0, model->getSampleRate());
+    RealTime rt1 = RealTime::frame2RealTime(f1, model->getSampleRate());
 
     if (f1 != f0 + 1 && (rt0.sec != rt1.sec || rt0.msec() != rt1.msec())) {
         text += tr("Time:\t%1 - %2")
@@ -1174,7 +1320,7 @@ WaveformLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) cons
     for (int ch = minChannel; ch <= maxChannel; ++ch) {
 
         RangeSummarisableTimeValueModel::RangeBlock ranges;
-        m_model->getSummaries(ch, f0, f1 - f0, ranges, blockSize);
+        model->getSummaries(ch, f0, f1 - f0, ranges, blockSize);
 
         if (ranges.empty()) continue;
         
@@ -1202,8 +1348,8 @@ WaveformLayer::getFeatureDescription(LayerGeometryProvider *v, QPoint &pos) cons
             max = double(imax)/10000;
         }
 
-        int db = int(AudioLevel::multiplier_to_dB(std::max(fabsf(range.min()),
-                                                           fabsf(range.max())))
+        int db = int(AudioLevel::voltage_to_dB(std::max(fabsf(range.min()),
+                                                        fabsf(range.max())))
                      * 100);
 
         if (!singleValue) {
@@ -1248,7 +1394,7 @@ WaveformLayer::getYForValue(const LayerGeometryProvider *v, double value, int ch
         break;
 
     case MeterScale:
-        vy = AudioLevel::multiplier_to_preview(value, m);
+        vy = int(dBscaleMeter(value, m));
         break;
 
     case dBScale:
@@ -1286,7 +1432,7 @@ WaveformLayer::getValueForY(const LayerGeometryProvider *v, int y, int &channel)
 
     int vy = my - y;
     double value = 0;
-    double thresh = -50.f;
+    double thresh = m_dBMin;
 
     switch (m_scale) {
 
@@ -1295,12 +1441,12 @@ WaveformLayer::getValueForY(const LayerGeometryProvider *v, int y, int &channel)
         break;
 
     case MeterScale:
-        value = AudioLevel::preview_to_multiplier(vy, m);
+        value = AudioLevel::fader_to_voltage(vy, m, AudioLevel::Scale::Preview);
         break;
 
     case dBScale:
         value = (-thresh * double(vy)) / m + thresh;
-        value = AudioLevel::dB_to_multiplier(value);
+        value = AudioLevel::dB_to_voltage(value);
         break;
     }
 
@@ -1311,20 +1457,19 @@ bool
 WaveformLayer::getYScaleValue(const LayerGeometryProvider *v, int y,
                               double &value, QString &unit) const
 {
-    int channel;
-
+    int channel = 0;
     value = getValueForY(v, y, channel);
 
     if (m_scale == dBScale || m_scale == MeterScale) {
 
-        double thresh = -50.f;
+        double thresh = m_dBMin;
         
         if (value > 0.0) {
-            value = 10.0 * log10(value);
+            value = AudioLevel::voltage_to_dB(value);
             if (value < thresh) value = thresh;
         } else value = thresh;
 
-        unit = "dBV";
+        unit = "dB";
 
     } else {
         unit = "V";
@@ -1337,7 +1482,7 @@ bool
 WaveformLayer::getYScaleDifference(const LayerGeometryProvider *v, int y0, int y1,
                                    double &diff, QString &unit) const
 {
-    int c0, c1;
+    int c0 = 0, c1 = 0;
     double v0 = getValueForY(v, y0, c0);
     double v1 = getValueForY(v, y1, c1);
 
@@ -1350,18 +1495,17 @@ WaveformLayer::getYScaleDifference(const LayerGeometryProvider *v, int y0, int y
 
     if (m_scale == dBScale || m_scale == MeterScale) {
 
-        double thresh = -50.0;
+        double thresh = m_dBMin;
 
         if (v1 == v0) diff = thresh;
         else {
             if (v1 > v0) diff = v0 / v1;
             else diff = v1 / v0;
-
-            diff = 10.0 * log10(diff);
+            diff = AudioLevel::voltage_to_dB(diff);
             if (diff < thresh) diff = thresh;
         }
 
-        unit = "dBV";
+        unit = "dB";
 
     } else {
         diff = fabs(v1 - v0);
@@ -1375,17 +1519,30 @@ int
 WaveformLayer::getVerticalScaleWidth(LayerGeometryProvider *, bool, QPainter &paint) const
 {
     if (m_scale == LinearScale) {
-        return paint.fontMetrics().width("0.0") + 13;
+        QString sampleText = "0.0";
+        if (m_gain != 1.f) {
+            int n = int(ceil(log10(m_gain)));
+            if (n > 2) {
+                sampleText = "0.0e+00";
+            } else {
+                while (n > 0) {
+                    sampleText += "0";
+                    --n;
+                }
+            }
+        }
+        return paint.fontMetrics().horizontalAdvance(sampleText) + 13;
     } else {
-        return std::max(paint.fontMetrics().width(tr("0dB")),
-                        paint.fontMetrics().width(Strings::minus_infinity)) + 13;
+        return std::max(paint.fontMetrics().horizontalAdvance(tr("0dB")),
+                        paint.fontMetrics().horizontalAdvance(Strings::minus_infinity)) + 13;
     }
 }
 
 void
 WaveformLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &paint, QRect rect) const
 {
-    if (!m_model || !m_model->isOK()) {
+    auto model = ModelById::getAs<RangeSummarisableTimeValueModel>(m_model);
+    if (!model || !model->isOK()) {
         return;
     }
 
@@ -1402,14 +1559,19 @@ WaveformLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pain
 
     double gain = m_gain;
 
+    int n = 10;
+
+    std::vector<ScaleTickIntervals::Tick> linearTicks;
+    if (m_scale == LinearScale) {
+        linearTicks = ScaleTickIntervals::linear({ 0.0, 1.0 / gain, n });
+    }
+
     for (int ch = minChannel; ch <= maxChannel; ++ch) {
 
         int lastLabelledY = -1;
 
         if (ch < (int)m_effectiveGains.size()) gain = m_effectiveGains[ch];
-
-        int n = 10;
-
+        
         for (int i = 0; i <= n; ++i) {
 
             double val = 0.0, nval = 0.0;
@@ -1418,17 +1580,31 @@ WaveformLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pain
             switch (m_scale) {
                 
             case LinearScale:
-                val = (i * gain) / n;
-                text = QString("%1").arg(double(i) / n);
-                if (i == 0) text = "0.0";
-                else {
-                    nval = -val;
-                    if (i == n) text = "1.0";
+                if (in_range_for(linearTicks, i)) {
+                    val = linearTicks[i].value * gain;
+                    text = QString::fromUtf8(linearTicks[i].label.c_str());
+                    if (i != 0) {
+                        nval = -val;
+                    }
+                } else {
+                    val = linearTicks[0].value * gain;
+                    text = QString::fromUtf8(linearTicks[0].label.c_str());
                 }
+                    
+                /*
+                val = double(i) / n;
+                text = QString("%1").arg(double(i) / (gain * n), 0, 'g', 2);
+//                if (i == 0) text = "0.0";
+//                else {
+                if (i != 0) {
+                    nval = -val;
+//                    if (i == n) text = "1.0";
+                }
+                */
                 break;
 
             case MeterScale:
-                val = AudioLevel::dB_to_multiplier(meterdbs[i]) * gain;
+                val = AudioLevel::dB_to_voltage(meterdbs[i]) * gain;
                 text = QString("%1").arg(meterdbs[i]);
                 if (i == n) text = tr("0dB");
                 if (i == 0) {
@@ -1438,7 +1614,7 @@ WaveformLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pain
                 break;
 
             case dBScale:
-                val = AudioLevel::dB_to_multiplier(-(10*n) + i * 10) * gain;
+                val = AudioLevel::dB_to_voltage(-(10*n) + i * 10) * gain;
                 text = QString("%1").arg(-(10*n) + i * 10);
                 if (i == n) text = tr("0dB");
                 if (i == 0) {
@@ -1464,7 +1640,7 @@ WaveformLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pain
 
                 int tx = 3;
                 if (m_scale != LinearScale) {
-                    tx = w - 10 - paint.fontMetrics().width(text);
+                    tx = w - 10 - paint.fontMetrics().horizontalAdvance(text);
                 }
                   
                 int ty = y;
@@ -1475,7 +1651,9 @@ WaveformLayer::paintVerticalScale(LayerGeometryProvider *v, bool, QPainter &pain
                 } else {
                     ty += toff;
                 }
-                paint.drawText(tx, ty, text);
+                if (!(i == n && ch > 0)) {
+                    paint.drawText(tx, ty, text);
+                }
 
                 lastLabelledY = ty - toff;
 
@@ -1521,7 +1699,7 @@ WaveformLayer::toXml(QTextStream &stream,
                  "scale=\"%6\" "
                  "middleLineHeight=\"%7\" "
                  "aggressive=\"%8\" "
-                 "autoNormalize=\"%9\"")
+                 "%9")
         .arg(m_gain)
         .arg(m_showMeans)
         .arg(true) // Option removed, but effectively always on, so
@@ -1531,13 +1709,16 @@ WaveformLayer::toXml(QTextStream &stream,
         .arg(m_scale)
         .arg(m_middleLineHeight)
         .arg(m_aggressive)
-        .arg(m_autoNormalize);
+        .arg(QString("autoNormalize=\"%1\" "
+                     "oversampling=\"%2\"")
+             .arg(m_autoNormalize)
+             .arg(m_oversampling));
 
     SingleColourLayer::toXml(stream, indent, extraAttributes + " " + s);
 }
 
 void
-WaveformLayer::setProperties(const QXmlAttributes &attributes)
+WaveformLayer::setProperties(const LayerAttributes &attributes)
 {
     bool ok = false;
 
@@ -1570,6 +1751,11 @@ WaveformLayer::setProperties(const QXmlAttributes &attributes)
     bool autoNormalize = (attributes.value("autoNormalize") == "1" ||
                           attributes.value("autoNormalize") == "true");
     setAutoNormalize(autoNormalize);
+
+    bool oversampling = (attributes.value("oversampling") == "" ||
+                         attributes.value("oversampling") == "1" ||
+                         attributes.value("oversampling") == "true");
+    setShowOversampledLines(oversampling);
 }
 
 int
@@ -1582,7 +1768,7 @@ WaveformLayer::getVerticalZoomSteps(int &defaultStep) const
 int
 WaveformLayer::getCurrentVerticalZoomStep() const
 {
-    int val = int(lrint(log10(m_gain) * 20.0) + 50);
+    int val = int(round(AudioLevel::voltage_to_dB(m_gain) + 50));
     if (val < 0) val = 0;
     if (val > 100) val = 100;
     return val;
@@ -1591,6 +1777,8 @@ WaveformLayer::getCurrentVerticalZoomStep() const
 void
 WaveformLayer::setVerticalZoomStep(int step)
 {
-    setGain(powf(10, float(step - 50) / 20.f));
+    setGain(AudioLevel::dB_to_voltage(step - 50));
 }
 
+} // end namespace sv
+
