@@ -21,11 +21,13 @@
 #include <QVBoxLayout>
 #include <QDialogButtonBox>
 #include <QApplication>
-#include <QDesktopWidget>
+#include <QScreen>
+
+namespace sv {
 
 KeyReference::KeyReference() :
-    m_text(0),
-    m_dialog(0)
+    m_text(nullptr),
+    m_dialog(nullptr)
 {
 }
 
@@ -45,22 +47,35 @@ KeyReference::setCategory(QString category)
 }
 
 void
-KeyReference::registerShortcut(QAction *action, QString overrideName)
+KeyReference::registerShortcut(QAction *action)
 {
     QString name = action->text();
-    if (overrideName != "") name = overrideName;
-
-    QString shortcut = action->shortcut().toString(QKeySequence::NativeText);
-    QString tip = action->statusTip();
-
-    registerShortcut(name, shortcut, tip);
+    name.replace(tr("&"), "");
+    registerShortcut(action, name);
 }
 
 void
-KeyReference::registerShortcut(QString name, QString shortcut, QString tip)
+KeyReference::registerShortcut(QAction *action, QString name)
+{
+    auto shortcuts = action->shortcuts();
+    int n = shortcuts.size();
+    if (n == 0) return;
+    registerShortcut(name, shortcuts[0], action->statusTip());
+    for (int i = 1; i < n; ++i) {
+        registerAlternativeShortcut(name, shortcuts[i]);
+    }
+}
+
+void
+KeyReference::registerShortcut(QString name, QKeySequence shortcut, QString tip)
+{
+    QString stext = shortcut.toString(QKeySequence::NativeText);
+    registerShortcutVerbatim(name, stext, tip);
+}
+
+void
+KeyReference::registerShortcutVerbatim(QString name, QString shortcut, QString tip)
 {
-    name.replace(tr("&"), "");
-            
     KeyList &list = m_map[m_currentCategory];
 
     for (KeyList::iterator i = list.begin(); i != list.end(); ++i) {
@@ -80,25 +95,32 @@ KeyReference::registerShortcut(QString name, QString shortcut, QString tip)
     list.push_back(details);
 }
 
-void
-KeyReference::registerAlternativeShortcut(QAction *action, QString alternative)
-{
-    QString name = action->text();
-    registerAlternativeShortcut(name, alternative);
-}
-
 void
 KeyReference::registerAlternativeShortcut(QAction *action, QKeySequence shortcut)
 {
     QString name = action->text();
-    registerAlternativeShortcut(name, shortcut.toString(QKeySequence::NativeText));
+    name.replace(tr("&"), "");
+    registerAlternativeShortcut(name, shortcut);
 }
 
 void
-KeyReference::registerAlternativeShortcut(QString name, QString alternative)
+KeyReference::registerAlternativeShortcutVerbatim(QAction *action, QString alternative)
 {
+    QString name = action->text();
     name.replace(tr("&"), "");
+    registerAlternativeShortcutVerbatim(name, alternative);
+}
 
+void
+KeyReference::registerAlternativeShortcut(QString name, QKeySequence shortcut)
+{
+    QString stext = shortcut.toString(QKeySequence::NativeText);
+    registerAlternativeShortcutVerbatim(name, stext);
+}
+
+void
+KeyReference::registerAlternativeShortcutVerbatim(QString name, QString alternative)
+{
     KeyList &list = m_map[m_currentCategory];
 
     for (KeyList::iterator i = list.begin(); i != list.end(); ++i) {
@@ -110,9 +132,33 @@ KeyReference::registerAlternativeShortcut(QString name, QString alternative)
 }
 
 void
-KeyReference::registerAlternativeShortcut(QString name, QKeySequence shortcut)
+KeyReference::makeMacMouseReplacements(QString &
+#ifdef Q_OS_MAC
+                                       mouseAction
+#endif
+    )
 {
-    registerAlternativeShortcut(name, shortcut.toString(QKeySequence::NativeText));
+#ifdef Q_OS_MAC
+    mouseAction.replace(tr("Ctrl+"), QChar(0x2318)); // Cmd
+    mouseAction.replace(tr("Shift+"), QChar(0x21E7)); // Shift
+    mouseAction.replace(tr("Alt+"), QChar(0x2325)); // Option
+    mouseAction.replace(tr("Right"), QChar(0x2303)); // Ctrl
+    mouseAction.replace(tr("Wheel"), tr("Scroll"));
+#endif
+}
+
+void
+KeyReference::registerMouseAction(QString name, QString mouseAction, QString tip)
+{
+    makeMacMouseReplacements(mouseAction);
+    registerShortcutVerbatim(name, mouseAction, tip);
+}
+
+void
+KeyReference::registerAlternativeMouseAction(QString name, QString mouseAction)
+{
+    makeMacMouseReplacements(mouseAction);
+    registerAlternativeShortcutVerbatim(name, mouseAction);
 }
 
 void
@@ -156,6 +202,9 @@ KeyReference::show()
                      k != j->alternatives.end(); ++k) {
                     QString alt = *k;
                     alt.replace(" ", "&nbsp;");
+                    if (k != j->alternatives.begin()) {
+                        altdesc += " ";
+                    }
                     altdesc += tr("<i>or</i>&nbsp;<b>%1</b>").arg(alt);
                 }
                 altdesc = tr("</b>&nbsp;(%1)<b>").arg(altdesc);
@@ -185,9 +234,9 @@ KeyReference::show()
     layout->addWidget(bb);
 
     m_dialog->show();
-    
-    QDesktopWidget *desktop = QApplication::desktop();
-    QRect available = desktop->availableGeometry();
+
+    QScreen *screen = QGuiApplication::primaryScreen();
+    QRect available = screen->availableGeometry();
 
     int width = available.width() * 3 / 5;
     int height = available.height() * 2 / 3;
@@ -216,3 +265,5 @@ KeyReference::hide()
         m_dialog->hide();
     }
 }
+} // end namespace sv
+
