@@ -26,10 +26,12 @@
 #include <QDialogButtonBox>
 #include <QScrollArea>
 #include <QApplication>
-#include <QDesktopWidget>
+#include <QScreen>
 #include <QTimer>
 #include <QAction>
 
+namespace sv {
+
 TransformFinder::TransformFinder(QWidget *parent) :
     QDialog(parent),
     m_resultsFrame(nullptr),
@@ -92,8 +94,8 @@ TransformFinder::TransformFinder(QWidget *parent) :
     connect(down, SIGNAL(triggered()), this, SLOT(down()));
     addAction(down);
 
-    QDesktopWidget *desktop = QApplication::desktop();
-    QRect available = desktop->availableGeometry();
+    QScreen *screen = QApplication::primaryScreen();
+    QRect available = screen->availableGeometry();
 
     int width = available.width() / 2;
     int height = available.height() / 2;
@@ -123,9 +125,9 @@ void
 TransformFinder::setupBeforeSearchLabel()
 {
     bool haveInstalled =
-        TransformFactory::getInstance()->haveInstalledTransforms();
+        TransformFactory::getInstance()->haveAnyInstalledTransforms();
     bool haveUninstalled =
-        TransformFactory::getInstance()->haveUninstalledTransforms();
+        TransformFactory::getInstance()->haveAnyUninstalledTransforms();
 
     m_beforeSearchLabel->setWordWrap(true);
     m_beforeSearchLabel->setOpenExternalLinks(true);
@@ -169,7 +171,7 @@ TransformFinder::setupBeforeSearchLabel()
 void
 TransformFinder::searchTextChanged(const QString &text)
 {
-//    cerr << "text is " << text << endl;
+//    SVCERR << "text is " << text << endl;
     m_newSearchText = text;
 }
 
@@ -183,11 +185,11 @@ TransformFinder::timeout()
         QString text = m_newSearchText;
         m_newSearchText = "";
 
-        QStringList keywords = text.split(' ', QString::SkipEmptyParts);
+        QStringList keywords = text.split(' ', Qt::SkipEmptyParts);
         TransformFactory::SearchResults results =
             TransformFactory::getInstance()->search(keywords);
         
-//        cerr << results.size() << " result(s)..." << endl;
+//        SVCERR << results.size() << " result(s)..." << endl;
         
         std::set<TextMatcher::Match> sorted;
         sorted.clear();
@@ -241,7 +243,7 @@ TransformFinder::timeout()
 
         int i = m_upToDateCount;
 
-//        cerr << "sorted size = " << m_sortedResults.size() << endl;
+//        SVCERR << "sorted size = " << m_sortedResults.size() << endl;
 
         TransformDescription desc;
         TransformId tid = m_sortedResults[i].key;
@@ -251,7 +253,7 @@ TransformFinder::timeout()
         QString suffix;
 
         if (status == TransformFactory::TransformInstalled) {
-            desc = factory->getTransformDescription(tid);
+            desc = factory->getInstalledTransformDescription(tid);
         } else {
             desc = factory->getUninstalledTransformDescription(tid);
             suffix = tr("<i> (not installed)</i>");
@@ -285,21 +287,53 @@ TransformFinder::timeout()
                 .arg(XmlExportable::encodeEntities(desc.description));
         }
 
+        QString indentation = tr("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&mdash;");
+        
         selectedText += tr("<small>");
         if (desc.type != TransformDescription::UnknownType) {
-            selectedText += tr("<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&mdash; Plugin type: %1")
-                .arg(XmlExportable::encodeEntities(factory->getTransformTypeName(desc.type)));
+            selectedText += tr("<br>%1 Plugin type: %2")
+                .arg(indentation)
+                .arg(XmlExportable::encodeEntities
+                     (factory->getTransformTypeName(desc.type)));
         }
         if (desc.category != "") {
-            selectedText += tr("<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&mdash; Category: %1")
+            selectedText += tr("<br>%1 Category: %2")
+                .arg(indentation)
                 .arg(XmlExportable::encodeEntities(desc.category));
         }
-        selectedText += tr("<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&mdash; System identifier: %1")
+        selectedText += tr("<br>%1 System identifier: %2")
+            .arg(indentation)
             .arg(XmlExportable::encodeEntities(desc.identifier));
-        if (desc.infoUrl != "") {
-            selectedText += tr("<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&mdash; More information: <a href=\"%1\">%1</a>")
-                .arg(desc.infoUrl);
+        if (desc.provider.infoUrl != "") {
+            selectedText += tr("<br>%1 More information: <a href=\"%2\">%2</a>")
+                .arg(indentation)
+                .arg(desc.provider.infoUrl);
         }
+        if (status != TransformFactory::TransformInstalled) {
+            bool haveSuitableDownloadLink =
+                (desc.provider.downloadUrl != "" &&
+                 desc.provider.hasDownloadForThisPlatform());
+            if (haveSuitableDownloadLink) {
+                selectedText += tr("<br>%1 Download for %2: <a href=\"%3\">%3</a>")
+                    .arg(indentation)
+                    .arg(desc.provider.thisPlatformName())
+                    .arg(desc.provider.downloadUrl);
+            }
+            if (!desc.provider.foundInPacks.empty()) {
+                QStringList packsLinks;
+                for (auto p: desc.provider.foundInPacks) {
+                    packsLinks <<
+                        tr("<a href=\"%1\">%2</a>").arg(p.second).arg(p.first);
+                }
+                selectedText +=
+                    (haveSuitableDownloadLink ?
+                     tr("<br>%1 Also available in: %2") :
+                     tr("<br>%1 Available in: %2"))
+                    .arg(indentation)
+                    .arg(packsLinks.join(tr(", ")));
+            }
+        }
+
         selectedText += tr("</small>");
 
         if (i >= (int)m_labels.size()) {
@@ -318,7 +352,7 @@ TransformFinder::timeout()
         m_labels[i]->setFixedWidth(this->width() - 40);
         m_labels[i]->setUnselectedText(labelText);
 
-//        cerr << "selected text: " << selectedText << endl;
+//        SVCERR << "selected text: " << selectedText << endl;
         m_labels[i]->setSelectedText(selectedText);
 
         m_labels[i]->setSelected(m_selectedTransform == desc.identifier);
@@ -350,7 +384,7 @@ TransformFinder::selectedLabelChanged()
             }
         }
     }
-    cerr << "selectedLabelChanged: selected transform is now \""
+    SVCERR << "selectedLabelChanged: selected transform is now \""
               << m_selectedTransform << "\"" << endl;
 }
 
@@ -404,3 +438,5 @@ TransformFinder::down()
     }
 }
 
+} // end namespace sv
+
